// CodeMirror, copyright (c) by Marijn Haverbeke and others
// Distributed under an MIT license: http://codemirror.net/LICENSE

describe('test wikidata cases', () => {
  const CodeMirror = require('codemirror');
  require('../index.js');
  const assert = require('assert');
  const Pos = CodeMirror.Pos;

  const namespace = "wikidata-hint_";

  testData = [
    {
      name: "simple test",
      value: "QMass-Energy equivalence",
      list: [
        {
          className: 'wikibase-rdf-hint',
          text: 'Q35875',
          displayText: 'mass–energy equivalence (Q35875) physical law: mass and energy are proportionate measures of the same underlying property of an object\n'
        }]
    }
  ];


  function escapeHtmlList(o) {
    return '<code>' +
      JSON.stringify(o.list, null, 2)
        .replace(/</g, "&lt;")
        .replace(/>/g, "&gt;") +
      '</code>'
      ;
  }

  testData.forEach(function(spec) {
    // Use sane defaults
    const lines = spec.value.split(/\n/);
    spec.to = spec.pos || Pos(lines.length - 1, lines[lines.length - 1].length);
    spec.from = spec.from || Pos(lines.length - 1, 0);
    spec.cursor = spec.cursor || spec.to;
    const name = spec.name || spec.value;
    it(namespace + name, () => {
      {
        const cm = CodeMirror(global.document.body);
        cm.setValue(spec.value);
        cm.setCursor(spec.cursor);
        return CodeMirror.hint.wikidata(cm).then(
          list => {
            assert.deepEqual(list.list, spec.list);
          });
      }
    });
  });


});
